;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x9422a7a45096426c427a9cec029bc24c . "./code.cg2js/cg-common-control.fasl")
(#x11bd83457bc1f9556c4195ec74ab8c16 . "./code.cg2js/cg-header-control.fasl")
(#x674fa2b7aad7f100000eb9ef48e591b9 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x784611236b18275328e368849dac54b1 . "./code.cg2js/cg-yes-no-string.fasl")
(#x2d56d99c4bd94292cc3860f669c0b187 . "./code.cg2js/cg-clipboard.fasl")
(#x25fa9d6e797077c81ec28e046fb8fe34 . "./code.cg2js/cg-dde-utils.fasl")
(#x8866b5c1e46178aa0aa04e3c3155855c . "./code.cg2js/cg-editable-text.fasl")
(#x6fb6856300af2a5ac2872ec4c8f35b5c . "./code.cg2js/cg-text-or-combo.fasl")
(#xfaaabddd42848e39506240ccff622a90 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x5e999c5cd74219efa7a83fa7ef4d8552 . "./code.cg2js/cg-graphics-context.fasl")
(#x889adc8125af2afa5e2228614a6920eb . "./code.cg2js/cg-choose-printer.fasl")
(#xe03d5ffe744d1a222772e4a404678a1f . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x3cc1cb0bf399840fb0a6fc5b1126aa63 . "./code.cg2js/cg-edit-in-place.fasl")
(#x60afca4d3e4d5b8026002e8f1fb8314e . "./code.cg2js/cg-split-bar.fasl")
(#x32c171e576ae8a6b0b6749a8f3f2e61a . "./code.cg2js/cg-timer.fasl")
(#xa25d029c315c2a7f4116ec22cd45c569 . "./code.cg2js/cg-progress-indicator.fasl")
(#xe475ad95e69395316abe3b9b83caccea . "./code.cg2js/cg-object-editor.fasl")
(#x41b6a05be7509db7a03a0ba1b3dd4c38 . "./code.cg2js/cg-fill-texture.fasl")
(#x3fd5e96c64c628b5b7956bc8d0ba617c . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x0e8967bade08847aa89c1ea253021ff3 . "./code.cg2js/cg-os-window.fasl")
(#xe14d018e77914109784ecedc0701ef38 . "./code.cg2js/cg-color-dialog.fasl")
(#x294e73843b7525b7bd3be851f663d3a1 . "./code.cg2js/cg-combo-box.fasl")
(#x10b55a62bbd1599daf6048e6343f6a7b . "./code.cg2js/cg-directory-dialog.fasl")
(#xcb1433b3c962887f64eddc59793a4192 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xb98c9ea6e08c68454d8f2ab56ac55122 . "./code.cg2js/cg-tooltip.fasl")
(#xdf93dcce9e23d43db9465e63c8c9d97d . "./code.cg2js/cg-lettered-menu.fasl")
(#xdb1310c6560006004d6af406e4927124 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x38ff070b5e5ee41677bfaacee57e0881 . "./code.cg2js/cg-pixmap.fasl")
(#x66c5a926849b6c5e6944d89316965a71 . "./code.cg2js/cg.fasl")
(#x73278140c4ae019f582a359f65812568 . "./code.cg2js/cg-class-slot-grid.fasl")
(#xc4b086d4d7ff0299a6168541dcbd4304 . "./code.cg2js/cg-template-string.fasl")
(#xd1043f9655c43bcc775f96bc912943ea . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xfbd696a6f312ce8402b4d6964133a74e . "./code.cg2js/cg-check-box.fasl")
(#x0659f641c935191d614b8344b405e176 . "./code.cg2js/cg-toggling-widget.fasl")
(#x33050695672c211b12659dd41a99ce9b . "./code.cg2js/cg-radio-button.fasl")
(#x9db46da2f34d422350030b1feb6fae40 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xf8867234b2fb814ce5a732012f797574 . "./code.cg2js/cg-scaling-stream.fasl")
(#xf8777ddb9474c7cbe62081007e61c5f3 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x910ef5a60d32641fd365f4573a4e0e08 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x5c886deb7d23bd7e432fc632e4ba31e9 . "./code.cg2js/cg-grid-widget.fasl")
(#x42a04380843d9fc0189ff626154660cb . "./code.cg2js/cg-text-edit-pane.fasl")
(#x3ef94a67f5479978fe2018df2f5cc0b7 . "./code.cg2js/cg-item-list.fasl")
(#xed3333d80af4616e6cb339d3110d9318 . "./code.cg2js/cg-paren-matching.fasl")
(#x277e9788c32ccc14f5abf595078795cb . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf374b8d2fddfe113aad94459b56c8fcc . "./code.cg2js/cg-html-dialog.fasl")
(#x90ad26272c0bd59d67f406ea784c6dc7 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x92ec6cd79c49db6d3014e12eaca10a02 . "./code.cg2js/cg-project-window.fasl")
(#xee8502bc9000fcc4a81ce615d31a8cfa . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x716badeb22a4f2a7fc2342b123c29d3d . "./code.cg2js/cg-toolbar.fasl")
(#x33b26b6b31a0df5e36c8bbb3129dc0d2 . "./code.cg2js/cg-caret.fasl")
(#x279adaeb4cc2932735f3ab69c59370bd . "./code.cg2js/cg-scroll-bar.fasl")
(#x8ad9d0c500a1a9435b80fc1e62dbc180 . "./code.cg2js/cg-web-browser.fasl")
(#xbc434e33cccf8dad2401826f73cb0577 . "./code.cg2js/cg-dropping-outline.fasl")
(#x6d48478be21f5d338f621e3201047bf9 . "./code.cg2js/cg-font-dialog.fasl")
(#xd05b6d18f8d6aa04af70af674815f1c8 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x59a14a22d586c978e3661ac7da3300aa . "./code.cg2js/cg-yes-no-list.fasl")
(#xf04205068b277b75d063c24a9a0fdfe8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xe34188bb21f14a036adaecd9ff09c5d7 . "./code.cg2js/cg-lamp.fasl")
(#xcd8a9650500b877906136da93bde4bdf . "./code.cg2js/cg-drawable.fasl")
(#x0a4ba7c39a0ffbb0ceb80b1365de0b6a . "./code.cg2js/cg-choice-list.fasl")
(#x6fd61bb4dd86f649dd0db877d8ee7707 . "./code.cg2js/cg-curve.fasl")
(#xcc28b17fd13ba95ce5fcec2be9eee335 . "./code.cg2js/cg-base.fasl")
(#x69ad08aeeec8c14c8880145a3305d765 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x6a95aaa0a68d54d8e7cabec628657551 . "./code.cg2js/cg-printing.fasl")
(#x9fc8e32c21bd607c881f579082f8a34e . "./code.cg2js/cg-chart-or-plot.fasl")
(#xac06944452d3278f192b42c5b1214555 . "./code.cg2js/cg-hotspot.fasl")
(#xb76c77a5a044c163c36449fe38cae036 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x49d22d784bfe330b17b0345995de32cf . "./code.cg2js/cg-utility-dialog.fasl")
(#x8ca12dbe1396f61613b32b912f037db1 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x1e06d75e0630916608b5a6ce4ca996e7 . "./code.cg2js/cg-comtab.fasl")
(#xce20c6ecfea18969716782df40f4a138 . "./code.cg2js/cg-os-widget.fasl")
(#x53b99983353ac3e4684b528359edd65a . "./code.cg2js/cg-icon.fasl")
(#x9ef13e90e6aec407da09ea75e274f5bb . "./code.cg2js/cg-dev.fasl")
(#xcec08fd72e83ec6956f06c7250888403 . "./code.cg2js/cg-calendar.fasl")
(#x7c3dafcc1662fe49ac8397930791cdb1 . "./code.cg2js/cg-group-box.fasl")
(#x378074125027ac3869b5703fc45b8b56 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xb65853707c12e4f3dc1cae2de071e5b5 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa9a3988f0ce6bd5116512e5e39c4897f . "./code.cg2js/cg-drag-and-drop.fasl")
(#x70f40d399990be2e4a6f60b0c55bcc15 . "./code.cg2js/cg-nodes.fasl")
(#xc835e82915fde12cd684bb04270aa42b . "./code.cg2js/cg-intersect.fasl")
(#x1ee5379700983a531a38227eaaee23b0 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x7d6cc8cc96e208746030a8a2f26f509f . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x9f6a8373d1905d9a962140992c90853b . "./code.cg2js/cg-class-support.fasl")
(#x78ef76a2f10e5429abca8a1d5585f241 . "./code.cg2js/cg-dialog-item.fasl")
(#x8b677005b4ea7dc4f5d809e1d4d0e9e6 . "./code.cg2js/cg-plot-widget.fasl")
(#xe6d2bb46a2e48a6aa8ec3d65102a5904 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x023e5abcc1376a66a28ee31a8b945002 . "./code.cg2js/cg-up-down-control.fasl")
(#xf5450501fa0ead18953e3dec0df6e950 . "./code.cg2js/cg-rich-edit.fasl")
(#x2279e721d9435d44717c683408bb89a7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x9b1ca7c978358b66ffb70da7479ae536 . "./code.cg2js/cg-status-bar.fasl")
(#xdd5facc6c99a5e3923f58bb68c3599e2 . "./code.cg2js/cg-class-grid.fasl")
(#xcb021012e4d58e53d7d44b8280d04909 . "./code.cg2js/cg-wrap-string.fasl")
(#x1c2c15fb7cc7291a503f2bada2fc3308 . "./code.cg2js/cg-layout.fasl")
(#x092a163de1dc5f02bc74c280719af209 . "./code.cg2js/cg-lisp-widget.fasl")
(#x1dd5b21f5a2642a5d9af90e4addc32e4 . "./code.cg2js/cg-button.fasl")
(#xb568ee7762c37ecce3478114225466cf . "./code.cg2js/cg-string-dialog.fasl")
(#x696052380399abc075197fccc211a2ee . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x8d19ca08b9a8ab66cc4f1cf60cd44713 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x4a018935d248315a16643bd103d18a10 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xafcbb3e00ad8250867f8e25154f1909f . "./code.cg2js/cg-static-text.fasl")
(#x6e98b1a0cbe9f2471d5027b19e3d4fe5 . "./code.cg2js/cg-outline.fasl")
(#x86c0f84a2dc42e1e4611eda6e77c3fba . "./code.cg2js/cg-gesture-emulation.fasl")
(#xbd640fde567b1f85739810ffd7dc97f6 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xbfde5c048980a025ea049976c3598d2d . "./code.cg2js/cg-picture-widget.fasl")
(#xbaa2976bcb19c8fa5d123679aecf90a7 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x46d8ab8519252daa016154ffe76766d5 . "./code.cg2js/cg-chart-widget.fasl")
(#xe9b440683ecfe454edaadb66c98e2fba . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xfa73aa64754cc5ca75d30e99f0070f3f . "./code.cg2js/cg-lisp-text.fasl")
(#x7f3c7cad1ae7638bc30b98af9afce093 . "./code.cg2js/cg-palette.fasl")
(#x9d2331bf8f71901c397a948de9f0cd05 . "./code.cg2js/cg-property.fasl")
(#x4438f0d3bd4e9e5e7e3b82950a3f1c10 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x8918c3a90b972fce982a839488f79e7a . "./code.cg2js/cg-file-dialog.fasl")
(#x443d210d17d58e78d815d338293aa31a . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xa54e1be9e7506656f0fcb787bfcdc0b1 . "./code.cg2js/cg-get-pixmap.fasl")
(#x3f293e5b51b86a7c1a56c1d405e6219e . "./code.cg2js/cg-tray.fasl")
(#x7f12a158f7ccd7e48fdcd196400087c9 . "./code.cg2js/cg-acache.fasl")
(#xcbc1699f822c1bb58fb4007d309452c0 . "./code.cg2js/cg-message-dialog.fasl")
(#xfb9f79f25b63b2c86f26e594721f1cad . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xfcb576465f7cf6b8bd45520cf3c0e2ac . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x024eee00e16896546807decaf5415d2a . "./code.cg2js/cg-tab-control.fasl")
(#x393bc46920b563e86997c17b235b037c . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x944cc53ea16b8da9a0a127ba80f72990 . "./code.cg2js/cg-menu-tooltip.fasl")
(#xa5fa56ba071dae2208a52dece3058e5e . "./code.cg2js/cg-find-string-dialog.fasl")
(#x96cb27770afc4d302821bcfaa3a63196 . "./code.cg2js/cg-menu.fasl")
(#x64cddedfbfc0d090c936d544d4df1dfd . "./code.cg2js/cg-get-position.fasl")
(#x01b987450b9cf59184f8ddefca5d8e86 . "./code.cg2js/cg-trackbar.fasl")
(#xbe698503004732485f55a6fdae0c9020 . "./code.cg2js/cg-selected-object.fasl")
(#x0138a0631303e18b0bd580953fd0f1e2 . "./code.cg2js/cg-list-view.fasl")
(#x1e5ef4438447af5b29c7c645f290f939 . "./code.cg2js/cg-html-widget.fasl")
(#xc377aae86fa1167dfc3628ea91cabbe7 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x77c5a2c6f37f10f8057237cd5f2b62fa . "./code.cg2js/cg-text-widget.fasl")
(#xf0954c02648d0039a60b1e4a5f940868 . "./code.idejs/ide-base.fasl")
(#x9cd7c0f7511cd55677208a7944aaf035 . "./code.idejs/ide-process-browser.fasl")
(#x14c9c66d24d9bf0c9f2053daa0121776 . "./code.idejs/ide-patch-dialog.fasl")
(#x74b1b368ddddf876a2b1e8dab0478bf4 . "./code.idejs/ide-keyshort.fasl")
(#x2aa8df47af95d1ff527842542e98b7b8 . "./code.idejs/ide-window-list.fasl")
(#x63d0e520d6e11b69dda2dfe159501642 . "./code.idejs/ide.fasl")
(#x03dc0955d02633f61954f6810cd6d335 . "./code.idejs/ide-editor.fasl")
(#xd6c9e64d45b8088e54323c771dee80e2 . "./code.idejs/ide-clipboard.fasl")
(#x3e9bacf47984b6f1fcc4a6639f7743a3 . "./code.idejs/ide-trace.fasl")
(#x51c374dab154b70624b5d69cfdec1b18 . "./code.idejs/ide-class-browser.fasl")
(#xf0f1b03781bc27835899509f936d1333 . "./code.idejs/ide-grapher.fasl")
(#x4d04262ceb256d63ead36939a51d10b2 . "./code.idejs/ide-apropos.fasl")
(#x4273d414b84b204ddadd9f4e9b660c95 . "./code.idejs/ide-navigator.fasl")
(#x059095a81f456ff571905170418019dd . "./code.idejs/ide-find-definitions.fasl")
(#x7608ab9257234635606504d5ca4b7b0f . "./code.idejs/ide-project.fasl")
(#x241246ec755fd8eb0d1b75a8d1909090 . "./code.idejs/ide-class-grapher.fasl")
(#x6132dfc9757f2c73b78c7dc184761aa7 . "./code.idejs/ide-acache.fasl")
(#xa310a4b4f046d62a6a5b4d85acc1a860 . "./code.idejs/ide-find-in-files.fasl")
(#x7392143992719582cce5405f9d0a7da2 . "./code.idejs/ide-inspector.fasl")
(#x6de96ea131c131894e4a63d99cc865ae . "./code.idejs/ide-intro.fasl")
(#x207354e0713f526239665e738da1987a . "./code.idejs/ide-package-list.fasl")
(#x125634648588181cb2a0e6c16ab15b12 . "./code.idejs/ide-dev.fasl")
(#x626eec3312f4efb489fbaf5b3cf568f5 . "./code.idejs/ide-form.fasl")
(#xeb5b70ef6a4ced03becd92b82d3b0061 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
